#include "stdafx.h"
#include <math.h>
#include "simple_app.h"
#include "clf_timer.h"

static TCHAR szTitle[100];					// The title bar text
static TCHAR szWindowClass[100];			// the main window class name

static HINSTANCE hInst;						// current instance
static HWND  hWnd=0;						// Window Handle
static HDC   hDC=0;							// Device Context
static HGLRC hRC=0;							// Rendering Context
static bool isMinimized = FALSE;			// is window visible or minimized

// windows stuff
static BOOL InitInstance(HINSTANCE, int);
static LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);

// our stuff
static void start_level();
static void frame_move(float fElapsed);
static void frame_render();
static void resize_window (int width, int height);
static HWND create_GL_window (int wid, int hei, int bitsPerPixel, const char* title, HINSTANCE hInstance, const char* classname, int stencilBuffer);
static bool destroy_GL_window ();

// animation variables
static float speed;					// speed of object
static float position[3];			// object position vector (x,y,z)

static bool keyDown [256];			// holds true / false for each key

// program entry point
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

	MSG msg;

	strcpy(szTitle, "GusOnGames");
	strcpy(szWindowClass, "GusOnGames");

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
	{
		return FALSE;
	}

	// zero the keystatus buffers
	memset(keyDown, 0, sizeof(keyDown));
	// start the timer before we enter the main loop
	get_global_timer()->start();
	// Main message loop:
	bool bLooping = true;
	start_level();
	while (bLooping)
	{
		// check for windows message and precess them
		if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE) != 0)
		{
			if (msg.message == WM_QUIT)
				bLooping = false;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else  // no messages, just loop for next frame
		{
			if (isMinimized)		// if window is minimized
			{
				WaitMessage ();		// yeld back to system, do not waste processing power
			}
			else
			{
				// update the scene based on the elapsed time since last update
				frame_move((float)get_global_timer()->get_elapsed_time());
				frame_render();		// render scene
				SwapBuffers (hDC);	// Swap Buffers (Double Buffering)
			}
		}
	}
	destroy_GL_window();
	// stop the clock
	get_global_timer()->stop();

	return (int) msg.wParam;
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;

	hInst = hInstance; // Store instance handle in our global variable

	WNDCLASSEX wcex;
	ZeroMemory (&wcex, sizeof (WNDCLASSEX));

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SIMPLE_APP));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_APPWORKSPACE);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_SIMPLE_APP);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	if (RegisterClassEx (&wcex) == 0)
	{
		MessageBox (HWND_DESKTOP, "RegisterClassEx Failed!", "Error", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}

	hWnd = create_GL_window (800, 600, 32, szTitle, hInstance, szWindowClass, 1);

	if (!hWnd)
		return FALSE;

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;

	switch (message)
	{
	case WM_COMMAND: // menu commands
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		switch (wmId)
		{
		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_SIZE:	// window size change
		switch (wParam)	// evaluate
		{
		case SIZE_MINIMIZED:	// minimized?
			isMinimized = true;	// set isMinimized to true
			return 0;

		case SIZE_MAXIMIZED:	// maximized?
			isMinimized = false;// set isMinimized to false
			resize_window (LOWORD (lParam), HIWORD (lParam));	// recalc view - LoWord=Width, HiWord=Height
			return 0;

		case SIZE_RESTORED:		// restored?
			isMinimized = false;// set isMinimized to false
			resize_window (LOWORD (lParam), HIWORD (lParam));	// recalc view - LoWord=Width, HiWord=Height
			return 0;
		}
		break;
	case WM_KEYDOWN:												// Update Keyboard Buffers For clf_keys Pressed
		if ((wParam >= 0) && (wParam <= 255))						// Is Key (wParam) In A Valid Range?
		{
			keyDown [wParam] = true;					// Set The Selected Key (wParam) To True
			return 0;												// Return
		}
		break;															// Break

	case WM_KEYUP:													// Update Keyboard Buffers For clf_keys Released
		if ((wParam >= 0) && (wParam <= 255))						// Is Key (wParam) In A Valid Range?
		{
			keyDown [wParam] = false;					// Set The Selected Key (wParam) To False
			return 0;												// Return
		}
		break;															// Break

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

void resize_window (int width, int height)								// reshape the window when it's moved or resized
{
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
	glMatrixMode (GL_PROJECTION);										// select the projection matrix
	glLoadIdentity ();													// reset the projection matrix
	gluPerspective (45.0f, (float)(width)/(float)(height), 1.0f, 1000.0f);	// calculate the aspect ratio of the window
	glMatrixMode (GL_MODELVIEW);										// select the modelview matrix
	glLoadIdentity ();													// reset the modelview matrix
}

// This function Creates Our OpenGL Window
HWND create_GL_window (int wid, int hei, int bitsPerPixel, const char* title, HINSTANCE hInstance, const char* classname, int stencilBuffer)
{
	DWORD windowStyle = WS_OVERLAPPEDWINDOW;				// define our window style
	DWORD windowExtendedStyle = WS_EX_APPWINDOW;			// define the window's extended style

	int width = wid;
	int height = hei;

	RECT windowRect = {0, 0, width, height};				// define our window coordinates

	// adjust window, account for window borders
	AdjustWindowRectEx (&windowRect, windowStyle, 0, windowExtendedStyle);

	// create the opengl window
	hWnd = CreateWindowEx (windowExtendedStyle,	// extended style
		classname,								// class name
		title,									// window title
		windowStyle,							// window style
		0, 0,									// window x,y position
		windowRect.right - windowRect.left,		// window width
		windowRect.bottom - windowRect.top,		// window height
		HWND_DESKTOP,							// desktop is window's parent
		0,										// no menu
		hInstance,								// pass the window instance
		0);

	if (hWnd == 0)								// was window creation a success?
	{
		return hWnd;							// if not return false
	}

	hDC = GetDC(hWnd);							// grab a device context for this window
	if (hDC == 0)								// did we get a device context?
	{
		// Failed
		DestroyWindow (hWnd);					// destroy the window
		hWnd = 0;								// zero the window handle
		return hWnd;							// return false
	}

	PIXELFORMATDESCRIPTOR pfd =								// pfd tells windows how we want things to be
	{
		sizeof (PIXELFORMATDESCRIPTOR),						// size of this pixel format descriptor
		1,													// version number
		PFD_DRAW_TO_WINDOW |								// format must support window
		PFD_SUPPORT_OPENGL |								// format must support opengl
		PFD_DOUBLEBUFFER,									// must support double buffering
		PFD_TYPE_RGBA,										// request an rgba format
		bitsPerPixel,										// select our color depth
		0, 0, 0, 0, 0, 0,									// color bits ignored
		0,													// no alpha buffer
		0,													// shift bit ignored
		0,													// no accumulation buffer
		0, 0, 0, 0,											// accumulation bits ignored
		16,													// 16bit z-buffer (depth buffer)  
		stencilBuffer,										// stencil buffer
		0,													// no auxiliary buffer
		PFD_MAIN_PLANE,										// main drawing layer
		0,													// reserved
		0, 0, 0												// layer masks ignored
	};

	GLuint PixelFormat = ChoosePixelFormat (hDC, &pfd);	// find a compatible pixel format
	if (PixelFormat == 0)								// did we find a compatible format?
	{
		// Failed
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	if (SetPixelFormat (hDC, PixelFormat, &pfd) == false)// try to set the pixel format
	{
		// Failed
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	hRC = wglCreateContext (hDC);						// try to get a rendering context
	if (hRC == 0)										// did we get a rendering context?
	{
		// Failed
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	// make the rendering context our current rendering context
	if (wglMakeCurrent (hDC, hRC) == false)				// failed
	{
		wglDeleteContext (hRC);							// delete the rendering context
		hRC = 0;										// zero the rendering context
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	ShowWindow (hWnd, SW_NORMAL);						// make the window visible
	isMinimized = false;								// set isMinimized to false

	resize_window (width, height);						// reshape our gl window
	UpdateWindow(hWnd);

	return hWnd;										// window creating was a success
}

bool destroy_GL_window ()								// destroy the opengl window & release resources
{
	if (hWnd != 0)										// does the window have a handle?
	{	
		if (hDC != 0)									// does the window have a device context?
		{
			wglMakeCurrent (hDC, 0);					// set the current active rendering context to zero
			if (hRC != 0)								// does the window have a rendering context?
			{
				wglDeleteContext (hRC);					// release the rendering context
				hRC = 0;								// zero the rendering context

			}
			ReleaseDC (hWnd, hDC);						// release the device context
			hDC = 0;									// zero the device context
		}
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
	}

	return true;										// return true
}

/////////////////////////////////////////////////////////////////////////////////////////////////////
struct vector{
	vector(){
		x = y = z = 0;
	}
	vector(float _x, float _y, float _z):x(_x), y(_y), z(_z){
	}
	vector(vector& v):x(v.x), y(v.y), z(v.z){
	}
	float x,y,z;
	vector& operator=(vector& v){
		x = v.x;
		y = v.y;
		z = v.z;
		return *this;
	}
};
class game_object
{
public:
	vector speed;
	vector position;
	vector size;

	enum state {active, inactive, hidden};
	state m_state;

	game_object(){
		m_state = active;
	}
	virtual ~game_object(){
	}

	void init(vector spd, vector pos, vector sz, state s){
		speed = spd;
		position = pos;
		size = sz;
		m_state = s;
	}

	void frame_move(float fElapsed){
		position.x += speed.x * fElapsed;
		position.y += speed.y * fElapsed;
		position.z += speed.z * fElapsed;
	}

	void render(){
		if (m_state == hidden)
			return;
		float dx = size.x/2;
		float dy = size.y/2;
		glPushMatrix();
		glTranslatef(position.x,position.y,position.z);	// move into position
		glColor3f(.9f, .6f, 0.3f);							// set the color
		glBegin(GL_QUADS);									// drawing using quads
			glVertex3f( -dx, dy, 0.0f);					// top
			glVertex3f(-dx,  -dy, 0.0f);					// bottom left
			glVertex3f( dx, -dy, 0.0f);					// bottom right
			glVertex3f( dx, dy, 0.0f);					// bottom right
		glEnd();											// finished drawing the triangle
		glPopMatrix();
	}

	bool intersects(game_object& ob){
		float w = size.x/2;
		float h = size.y/2;
		float w1 = ob.size.x/2;
		float h1 = ob.size.y/2;
		float dx = fabs(position.x - ob.position.x);
		float dy = fabs(position.y - ob.position.y);
		if (dx < w+w1 && dy < h+h1)
			return true;
		return false;
	}
};

class ball : public game_object 
{
public:
	ball() : game_object(){
	}
	virtual ~ball(){
	}
};

class batt : public game_object
{
public:
	batt() : game_object(){
	}
	virtual ~batt(){
	}
};

game_object top_wall;
game_object left_wall;
game_object right_wall;
ball theBall;
batt theBatt;

void start_level()
{
	theBatt.init(vector(0,0,0), vector(0, -30, -80), vector(10,1,0), game_object::active);
	theBall.init(vector(5,10,0), vector(0, -28, -80), vector(1,1,0), game_object::active);
	top_wall.init(vector(0,0,0), vector(0, 30, -80), vector(66,1,0), game_object::active);
	left_wall.init(vector(0,0,0), vector(-33, 0, -80), vector(1,60,0), game_object::active);
	right_wall.init(vector(0,0,0), vector(33, 0, -80), vector(1,60,0), game_object::active);
}

void collision_detection()
{
	if (theBall.intersects(top_wall))
		theBall.speed.y = -theBall.speed.y;
	if (theBall.intersects(left_wall))
		theBall.speed.x = -theBall.speed.x;
	if (theBall.intersects(right_wall))
		theBall.speed.x = -theBall.speed.x;
	if (theBall.intersects(theBatt))
		theBall.speed.y = -theBall.speed.y;
}

void frame_move(float fElapsed)
{
	theBatt.speed.x = 0.f;
	if (keyDown[VK_ESCAPE]) 
		DestroyWindow(hWnd);
	if (keyDown[VK_LEFT]) // left
		theBatt.speed.x = -20.f;
	if (keyDown[VK_RIGHT]) // right
		theBatt.speed.x = 20.f;
	if (keyDown['R']) // restart
		start_level();

	theBall.frame_move(fElapsed);
	theBatt.frame_move(fElapsed);

	collision_detection();
}

void frame_render()
{
	glClearColor (0.0f, .0f, 0.0f, 1.f);				// black background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and depth buffer
	glLoadIdentity();									// reset the current modelview matrix

	top_wall.render();
	left_wall.render();
	right_wall.render();

	theBatt.render();
	theBall.render();
}
